@@ -14,41 +14,43 @@ from sutekh.gui.CardListView import CardListView
 from sutekh.gui.CardSetListModel import CardSetCardListModel
 from sutekh.core.SutekhObjects import PhysicalCardSet
 
-_dNumKeys = {
-        gtk.gdk.keyval_from_name('1') : 1,
-        gtk.gdk.keyval_from_name('KP_1') : 1,
-        gtk.gdk.keyval_from_name('2') : 2,
-        gtk.gdk.keyval_from_name('KP_2') : 2,
-        gtk.gdk.keyval_from_name('3') : 3,
-        gtk.gdk.keyval_from_name('KP_3') : 3,
-        gtk.gdk.keyval_from_name('4') : 4,
-        gtk.gdk.keyval_from_name('KP_4') : 4,
-        gtk.gdk.keyval_from_name('5') : 5,
-        gtk.gdk.keyval_from_name('KP_5') : 5,
-        gtk.gdk.keyval_from_name('6') : 6,
-        gtk.gdk.keyval_from_name('KP_6') : 6,
-        gtk.gdk.keyval_from_name('7') : 7,
-        gtk.gdk.keyval_from_name('KP_7') : 7,
-        gtk.gdk.keyval_from_name('8') : 8,
-        gtk.gdk.keyval_from_name('KP_8') : 8,
-        gtk.gdk.keyval_from_name('9') : 9,
-        gtk.gdk.keyval_from_name('KP_9') : 9
+NUM_KEYS = {
+        gtk.gdk.keyval_from_name('1'): 1,
+        gtk.gdk.keyval_from_name('KP_1'): 1,
+        gtk.gdk.keyval_from_name('2'): 2,
+        gtk.gdk.keyval_from_name('KP_2'): 2,
+        gtk.gdk.keyval_from_name('3'): 3,
+        gtk.gdk.keyval_from_name('KP_3'): 3,
+        gtk.gdk.keyval_from_name('4'): 4,
+        gtk.gdk.keyval_from_name('KP_4'): 4,
+        gtk.gdk.keyval_from_name('5'): 5,
+        gtk.gdk.keyval_from_name('KP_5'): 5,
+        gtk.gdk.keyval_from_name('6'): 6,
+        gtk.gdk.keyval_from_name('KP_6'): 6,
+        gtk.gdk.keyval_from_name('7'): 7,
+        gtk.gdk.keyval_from_name('KP_7'): 7,
+        gtk.gdk.keyval_from_name('8'): 8,
+        gtk.gdk.keyval_from_name('KP_8'): 8,
+        gtk.gdk.keyval_from_name('9'): 9,
+        gtk.gdk.keyval_from_name('KP_9'): 9,
         }
 
-_aPlusKeys = [
+PLUS_KEYS = set([
         gtk.gdk.keyval_from_name('plus'),
-        gtk.gdk.keyval_from_name('KP_Add')
-        ]
-_aMinusKeys = [
+        gtk.gdk.keyval_from_name('KP_Add'),
+        ])
+MINUS_KEYS = set([
         gtk.gdk.keyval_from_name('minus'),
-        gtk.gdk.keyval_from_name('KP_Subtract')
-        ]
+        gtk.gdk.keyval_from_name('KP_Subtract'),
+        ])
 
 
 class CardSetView(CardListView):
-    # pylint: disable-msg=R0904, R0902
+    # pylint: disable-msg=R0904, R0902, R0901
     # R0904 - gtk.Widget, so many public methods
     # R0902 - We need to track a fair amount of state, so many attributes
+    # R0901 - many ancestors, due to our object hierachy on top of the quite
+    # deep gtk one
     """Subclass of CardListView specific to the Card Sets
 
        Adds editing support, and other specific to the card sets.
@@ -60,8 +62,11 @@ class CardSetView(CardListView):
     # Initialise key ranges for key tests
     # pylint: disable-msg=R0915
     # We need a lot of setup here, so this is long
-    def __init__(self, oMainWindow, oController, sName):
-        oModel = CardSetCardListModel(sName)
+    def __init__(self, oMainWindow, oController, sName, bStartEditable):
+        oModel = CardSetCardListModel(sName, oMainWindow.config_file)
+        oModel.enable_sorting()
+        if bStartEditable:
+            oModel.bEditable = True
         # The only path here is via the main window, so config_file exists
         super(CardSetView, self).__init__(oController, oMainWindow,
                 oModel, oMainWindow.config_file)
@@ -73,7 +78,7 @@ class CardSetView(CardListView):
         self.oNumCell = gtk.CellRendererText()
         self.oNameCell = CellRendererIcons(5)
 
-        oColumn1 = gtk.TreeViewColumn("#", self.oNumCell, markup=1)
+        oColumn1 = gtk.TreeViewColumn("#", self.oNumCell, text=1)
         oColumn1.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
         oColumn1.set_fixed_width(60)
         oColumn1.set_sort_column_id(1)
@@ -81,8 +86,8 @@ class CardSetView(CardListView):
         self.append_column(oColumn1)
 
         oParentCell = gtk.CellRendererText()
-        self.oParentCol = gtk.TreeViewColumn("Par #", oParentCell,
-                markup=2)
+        self.oParentCol = gtk.TreeViewColumn("Par #", oParentCell, text=2,
+                foreground_gdk=7)
         self.oParentCol.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
         self.oParentCol.set_fixed_width(60)
         self.oParentCol.set_sort_column_id(2)
@@ -101,27 +106,29 @@ class CardSetView(CardListView):
         self.set_expander_column(oColumn2)
 
         # Inc/Dec cells
-        oCell3 = CellRendererSutekhButton()
-        oCell3.load_icon(gtk.STOCK_ADD, self)
-        oCell4 = CellRendererSutekhButton()
-        oCell4.load_icon(gtk.STOCK_REMOVE, self)
+        oIncCell = CellRendererSutekhButton()
+        oIncCell.load_icon(gtk.STOCK_ADD, self)
+        oDecCell = CellRendererSutekhButton()
+        oDecCell.load_icon(gtk.STOCK_REMOVE, self)
 
-        oColumn3 = gtk.TreeViewColumn("", oCell3, showicon=3)
-        oColumn3.set_fixed_width(22)
-        oColumn3.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
-        oColumn3.set_resizable(False)
-        self.append_column(oColumn3)
+        self.oIncCol = gtk.TreeViewColumn("", oIncCell, showicon=3)
+        self.oIncCol.set_fixed_width(19)
+        self.oIncCol.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+        self.oIncCol.set_resizable(False)
+        self.oIncCol.set_visible(False)
+        self.append_column(self.oIncCol)
 
-        oColumn4 = gtk.TreeViewColumn("", oCell4, showicon=4)
-        oColumn4.set_fixed_width(22)
-        oColumn4.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
-        oColumn4.set_resizable(False)
-        self.append_column(oColumn4)
+        self.oDecCol = gtk.TreeViewColumn("", oDecCell, showicon=4)
+        self.oDecCol.set_fixed_width(19)
+        self.oDecCol.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+        self.oDecCol.set_resizable(False)
+        self.oDecCol.set_visible(False)
+        self.append_column(self.oDecCol)
 
-        oCell3.connect('clicked', self.inc_card)
-        oCell4.connect('clicked', self.dec_card)
+        oIncCell.connect('clicked', self.inc_card)
+        oDecCell.connect('clicked', self.dec_card)
 
-        self.__iMapID = self.connect('map-event', self.mapped)
+        self.__iMapID = self.connect('map', self.mapped)
         self.connect('key-press-event', self.key_press)
 
         self._oMenu = None
@@ -129,11 +136,6 @@ class CardSetView(CardListView):
         self.oCellColor = None
 
         self.set_fixed_height_mode(True)
-        # Set sort functions so numerical sorts work as expected
-        # Card numbers
-        self._oModel.set_sort_func(1, self._oModel.num_col_sort_func, 1)
-        # Parent counts
-        self._oModel.set_sort_func(2, self._oModel.num_col_sort_func, 2)
 
     # pylint: enable-msg=R0915
 
@@ -182,131 +184,150 @@ class CardSetView(CardListView):
                 dSelectedData[sCardName][sExpansion] = iCount
         return dSelectedData
 
-    def process_edit_selection(self):
+    def _process_edit_selection(self, iSetNewCount=None, iChg=None):
         """Create a dictionary from the selection, suitable for the quick
            key based edits.
 
            Entries are of the form
-               sCardName : { (sExpansion1, sCardSet) : iCount1, ... }
+               oPhysCard : { sCardSetName : [iCount1, iNewCnt1] ... }
            In addition to adding information about card sets, this
            differs from process_selection in the way card level items
            are handled. Here, these are treated as selecting the expansion
            None, and ignoring other expansions.
+
+           Since this isn't going through the drag-n-drop code, we don't
+           need to create a string representation, so we can work with the
+           card objects directly.
+
            """
         oModel, oPathList = self._oSelection.get_selected_rows()
         dSelectedData = {}
+        iNewCount = 0
         for oPath in oPathList:
             sCardSet = None
-            sCardName, sExpansion, iCount, iDepth = \
-                    oModel.get_drag_info_from_path(oPath)
-            if not sCardName:
-                sCardName, sExpansion, sCardSet = \
-                        oModel.get_all_names_from_path(oPath)
-            dSelectedData.setdefault(sCardName, {})
+            oIter = oModel.get_iter(oPath)
+            oPhysCard = oModel.get_physical_card_from_iter(oIter)
+            iCount = oModel.get_card_count_from_iter(oIter)
+            if iChg:
+                iNewCount = max(0, iCount + iChg)
+            elif iSetNewCount:
+                iNewCount = iSetNewCount
+            _sCardName, _sExpansion, sCardSet = \
+                        oModel.get_all_names_from_iter(oIter)
+            dSelectedData.setdefault(oPhysCard, {})
+            iDepth = oModel.iter_depth(oIter)
             if iDepth == 1:
                 # this is treated as selecting all the entries of this card
                 # in this card set
                 # Remove anything already assigned to this
-                dSelectedData[sCardName].clear()
-                dSelectedData[sCardName][('All', None)] = iCount
+                dSelectedData[oPhysCard].clear()
+                dSelectedData[oPhysCard][None] = [iCount, iNewCount]
             else:
-                if (sExpansion, sCardSet) in dSelectedData[sCardName] or \
-                        ('All', None) in dSelectedData[sCardName]:
-                    # We already have this info
+                if sCardSet in dSelectedData[oPhysCard]:
+                    # We already have this info (for selections via multiple
+                    # groups, etc.)
                     continue
-                dSelectedData[sCardName][(sExpansion, sCardSet)] = iCount
+                dSelectedData[oPhysCard][sCardSet] = [iCount,
+                        iNewCount]
         return dSelectedData
 
-
-    # pylint: disable-msg=R0913, W0613
+    # pylint: disable-msg=R0913
     # elements required by function signature
     def card_drop(self, oWidget, oContext, iXPos, iYPos, oData, oInfo, oTime):
         """Handle drag-n-drop events."""
+        bDragRes = True
         if not oData or oData.format != 8:
             # Don't accept invalid data
-            oContext.finish(False, False, oTime)
+            bDragRes = False
         else:
             sSource, aCardInfo = self.split_selection_data(oData.data)
             bSkip = False
             if sSource == "Sutekh Pane:" or sSource == "Card Set:":
                 self._oController.frame.drag_drop_handler(oWidget, oContext,
                         iXPos, iYPos, oData, oInfo, oTime)
+                return
             elif not self._oModel.bEditable:
                 # Don't accept cards when not editable
                 bSkip = True
             elif sSource == self.sDragPrefix:
                 # Can't drag to oneself
                 bSkip = True
-            if not bSkip and self._oController.add_paste_data(sSource,
+            if bSkip or not self._oController.add_paste_data(sSource,
                     aCardInfo):
-                oContext.finish(True, False, oTime) # paste successful
-            else:
-                oContext.finish(False, False, oTime) # not successful
+                bDragRes = False  # paste failed
+            # else paste succeeds
+        oContext.finish(bDragRes, False, oTime)
 
     # pylint: enable-msg=R0913
 
-    # pylint: disable-msg=W0613
-    # arguments as required by the function signature
-    def inc_card(self, oCell, oPath):
+    def inc_card(self, _oCell, oPath):
         """Called to increment the count for a card."""
+        # We ignore the 'invalid path' failure case, since nothing should be
+        # able to remove the row out from under us.
         if self._oModel.bEditable:
-            # pylint: disable-msg=W0612
-            # only interested in bInc
-            bInc, bDec = self._oModel.get_inc_dec_flags_from_path(oPath)
+            bInc, _bDec = self._oModel.get_inc_dec_flags_from_path(oPath)
             if bInc:
-                sCardName, sExpansion, sCardSetName = \
+                oPhysCard = self._oModel.get_physical_card_from_path(oPath)
+                _sCardName, _sExpansion, sCardSetName = \
                         self._oModel.get_all_names_from_path(oPath)
-                self._oController.inc_card(sCardName, sExpansion, sCardSetName)
+                self._oController.inc_card(oPhysCard, sCardSetName)
 
-    def dec_card(self, oCell, oPath):
+    def dec_card(self, _oCell, oPath):
         """Called to decrement the count for a card"""
+        # Same assumption as inc_card about invalid path
         if self._oModel.bEditable:
-            # pylint: disable-msg=W0612
-            # only interested in bDec
-            bInc, bDec = self._oModel.get_inc_dec_flags_from_path(oPath)
+            _bInc, bDec = self._oModel.get_inc_dec_flags_from_path(oPath)
             if bDec:
-                sCardName, sExpansion, sCardSetName = \
+                oPhysCard = self._oModel.get_physical_card_from_path(oPath)
+                _sCardName, _sExpansion, sCardSetName = \
                         self._oModel.get_all_names_from_path(oPath)
-                self._oController.dec_card(sCardName, sExpansion, sCardSetName)
+                self._oController.dec_card(oPhysCard, sCardSetName)
 
-    def key_press(self, oWidget, oEvent):
+    def key_press(self, _oWidget, oEvent):
         """Change the number if 1-9 is pressed and we're editable or if + or
            - is pressed. We use the lists defined above to handle the keypad
            as well."""
-        if oEvent.keyval in _dNumKeys:
+        if oEvent.keyval in NUM_KEYS:
             if self._oModel.bEditable:
-                iCnt = _dNumKeys[oEvent.keyval]
-                dSelectedData = self.process_edit_selection()
-                self._oController.set_selected_card_count(dSelectedData, iCnt)
+                iCnt = NUM_KEYS[oEvent.keyval]
+                dSelectedData = self._process_edit_selection(iSetNewCount=iCnt)
+                self._oController.change_selected_card_count(dSelectedData)
             # if we're not editable, we still ignore this, so we avoid funny
             # search behaviour
             return True
-        elif oEvent.keyval in _aPlusKeys and self._oModel.bEditable:
-            dSelectedData = self.process_edit_selection()
-            self._oController.alter_selected_card_count(dSelectedData, 1)
+        elif oEvent.keyval in PLUS_KEYS and self._oModel.bEditable:
+            # Special value to indicate we change the count
+            dSelectedData = self._process_edit_selection(iChg=+1)
+            self._oController.change_selected_card_count(dSelectedData)
             return True
-        elif oEvent.keyval in _aMinusKeys and self._oModel.bEditable:
-            dSelectedData = self.process_edit_selection()
-            self._oController.alter_selected_card_count(dSelectedData, -1)
+        elif oEvent.keyval in MINUS_KEYS and self._oModel.bEditable:
+            # Special value to indicate we change the count
+            dSelectedData = self._process_edit_selection(iChg=-1)
+            self._oController.change_selected_card_count(dSelectedData)
             return True
-        return False # propogate event
+        return False  # propogate event
 
     # functions related to tweaking widget display
 
-    def mapped(self, oWidget, oEvent):
+    def mapped(self, _oWidget):
         """Called when the view has been mapped, so we can twiddle the
-           display"""
-        # see if we need to be editable
-        self.check_editable()
+           display
+
+           In the case when a card set is opened editable, we need to
+           load after the pane is mapped, so that the colours are setup
+           correctly. We also use the opportunity to ensure the menu
+           is in sync."""
         if self._oModel.bEditable:
-            # Ensure hint colour is set correctly
-            self.reload_keep_expanded()
+            self._set_editable(True)
         # We only ever need to call this the first time we're mapped.
         # We don't want to redo this if map is called again due to
         # panes moving, etc.
         self.disconnect(self.__iMapID)
-
-    # pylint: enable-msg=W0613
+        self.__iMapID = None
+        self.reload_keep_expanded()
+        # Allow other map signals to run as well (needed for drag-n-drop in
+        # some gtk versions)
+        return True
 
     # Anything that touches the database is based off to the controller
     # We handle the editable checks here though, since the controller methods
@@ -318,11 +339,12 @@ class CardSetView(CardListView):
     def del_selection(self):
         """try to delete all the cards in the current selection"""
         if self._oModel.bEditable:
-            dSelectedData = self.process_selection()
-            self._oController.del_selected_cards(dSelectedData)
+            dSelectedData = self._process_edit_selection(iSetNewCount=0)
+            self._oController.change_selected_card_count(dSelectedData)
 
     def do_paste(self):
-        """Try and paste the current selection from the appliction clipboard"""
+        """Try and paste the current selection from the application
+           clipboard"""
         if self._oModel.bEditable:
             sSelection = self._oMainWin.get_selection_text()
             sSource, aCards = self.split_selection_data(sSelection)
@@ -332,30 +354,25 @@ class CardSetView(CardListView):
 
     def load(self):
         """Called when the model needs to be reloaded."""
+        if not self.__iMapID is None:
+            # skip loading until we're mapped, to save double loads in
+            # some cases
+            return
         if hasattr(self._oMainWin, 'set_busy_cursor'):
             self._oMainWin.set_busy_cursor()
-        if self._oModel.get_card_iterator(None).count() == 0:
-            self._oModel.bEditable = True
-            # We do this before loading, so edit icons are correct
         self.freeze_child_notify()
         self.set_model(None)
         self._oModel.load()
-        self.check_editable()
         self.set_model(self._oModel)
+        self.oNumCell.set_property('foreground-gdk',
+                self._oModel.get_count_colour())
         self.thaw_child_notify()
         if hasattr(self._oMainWin, 'restore_cursor'):
             self._oMainWin.restore_cursor()
 
-    def check_editable(self):
-        """Set the card list to be editable if it's empty"""
-        if self.get_parent() and \
-                self._oModel.get_card_iterator(None).count() == 0:
-            # This isn't true when creating the view
-            self._set_editable(True)
-
     def set_color_edit_cue(self):
         """Set a visual cue that the card set is editable."""
-        if not self._oModel.sEditColour:
+        if not self._oModel.oEditColour:
             self._determine_edit_colour()
         self.set_name('editable_view')
 
@@ -391,7 +408,7 @@ class CardSetView(CardListView):
                 fg[NORMAL] = "%(colour)s"
                 }
             widget "%(path)s" style "internal_sutekh_editstyle"
-            """ % { 'colour' : sColour, 'path' : self.path() }
+            """ % {'colour': sColour, 'path': self.path()}
             gtk.rc_parse_string(sStyleInfo)
             # Need to force re-assesment of styles
             self.set_name('editable_view')
@@ -402,7 +419,7 @@ class CardSetView(CardListView):
         if not _compare_colors(oEditColor, self.oCellColor) or \
                 not _compare_colors(oEditBackColor, oCurBackColor):
             # Visiually distinct, so honour user's choice
-            self._oModel.sEditColour = oEditColor.to_string()
+            self._oModel.oEditColour = oEditColor
         else:
             # If the theme change isn't visually distinct here, we go
             # with red  as the default - this is safe,
@@ -410,7 +427,7 @@ class CardSetView(CardListView):
             # themed, so the default color will not be red
             # (famous last words)
             # If the default background color is red, too bad
-            self._oModel.sEditColour = 'red'
+            self._oModel.oEditColour = gtk.gdk.color_parse('red')
 
     def set_color_normal(self):
         """Unset the editable visual cue"""
@@ -423,8 +440,12 @@ class CardSetView(CardListView):
             self._oMenu.force_editable_mode(bValue)
         if bValue:
             self.set_color_edit_cue()
+            self.oIncCol.set_visible(True)
+            self.oDecCol.set_visible(True)
         else:
             self.set_color_normal()
+            self.oIncCol.set_visible(False)
+            self.oDecCol.set_visible(False)
 
     def toggle_editable(self, bValue):
         """Reload the view and update status when editable status changes"""
@@ -445,3 +466,31 @@ class CardSetView(CardListView):
            etc."""
         self.sSetName = sNewName
         self.sDragPrefix = PhysicalCardSet.sqlmeta.table + ":" + self.sSetName
+
+    def save_iter_state(self, aIters, dStates):
+        """Save the expanded state of the list of iters in aIters and their
+           children."""
+        dStates.setdefault('expanded', set())
+        dStates.setdefault('selected', set())
+        for oIter in aIters:
+            sKey = self.get_iter_identifier(oIter)
+            if self._oSelection.iter_is_selected(oIter):
+                dStates['selected'].add(sKey)
+            oPath = self._oModel.get_path(oIter)
+            if self.row_expanded(oPath):
+                dStates['expanded'].add(sKey)
+            aChildIters = self._oModel.get_all_iter_children(oIter)
+            self.save_iter_state(aChildIters, dStates)
+
+    def restore_iter_state(self, aIters, dStates):
+        """Restore expanded state of the iters."""
+        if not dStates or not dStates.has_key('selected'):
+            return  # Don't do anything if dStates is empty
+        for oIter in aIters:
+            sKey = self.get_iter_identifier(oIter)
+            if sKey in dStates['selected']:
+                self._oSelection.select_iter(oIter)
+            if sKey in dStates['expanded']:
+                self.expand_to_path(self._oModel.get_path(oIter))
+            aChildIters = self._oModel.get_all_iter_children(oIter)
+            self.restore_iter_state(aChildIters, dStates)
