@@ -32,6 +32,8 @@ except SQLObjectNotFound, oExcDetails:
     raise ImportError('Unable to load the required expansions')
 ODD_BACKS = (None, THIRD_ED, JYHAD)
 
+PDF_SETS = set()
+
 UNSLEEVED = "<span foreground='green'>%s may be played unsleeved</span>\n"
 SLEEVED = "<span foreground='orange'>%s should be sleeved</span>\n"
 SPECIAL = set(['Not Tournament Legal Cards', 'Multirole', 'Mixed Card Backs'])
@@ -70,6 +72,17 @@ def _disc_sort_cmp(oTuple1, oTuple2):
     return cmp(oTuple1[0].fullname, oTuple2[0].fullname)
 
 
+def _load_pdf_sets():
+    """Cache the sets from the VEKN pdf expansions"""
+    PDF_SETS.clear()
+    for sSet in ['Danse Macabre']:
+        try:
+            oSet = IExpansion(sSet)
+            PDF_SETS.add(oSet)
+        except SQLObjectNotFound:
+            pass
+
+
 def _format_card_line(sString, sTrailer, iNum, iLibSize):
     """Format card lines for notebook"""
     sPer = _percentage(iNum, iLibSize, "Library")
@@ -241,7 +254,9 @@ def _get_back_counts(aPhysCards):
     dCounts = {}
     for oCard in aPhysCards:
         oKey = oCard.expansion
-        if oKey not in ODD_BACKS:
+        if oKey in PDF_SETS:
+            oKey = 'PDF'
+        elif oKey not in ODD_BACKS:
             oKey = 'Other'
         dCounts.setdefault(oKey, 0)
         dCounts[oKey] += 1
@@ -272,13 +287,19 @@ def _check_same(aPhysCards):
     dLib = _get_back_counts(aLib)
     if len(dCrypt) > 1 or len(dLib) > 1:
         return False  # Differing backs
+    if 'PDF' in dCrypt or 'PDF' in dLib:
+        return False  # PDF set in use, so must be sleeved
     return True
 
 
 def _simple_back_checks(dCards, sType):
     """Simple checks on the card backs. Returns None if we need
        further investigation"""
-    if len(dCards) == 1 and not dCards.has_key(None):
+    if "PDF" in dCards:
+        sText = "%s has cards from a V:EKN pdf expansion.\n" % sType
+        sText += SLEEVED % sType
+        return sText
+    elif len(dCards) == 1 and not dCards.has_key(None):
         sText = "All %s cards have identical backs\n" % sType.lower() + \
                     UNSLEEVED % sType
         return sText
@@ -391,6 +412,10 @@ class AnalyzeCardList(SutekhPlugin):
     dTableVersions = {PhysicalCardSet: (6,)}
     aModelsSupported = (PhysicalCardSet,)
 
+    def __init__(self, *args, **kwargs):
+        super(AnalyzeCardList, self).__init__(*args, **kwargs)
+        _load_pdf_sets()
+
     def get_menu_item(self):
         """Register on the 'Analyze' Menu"""
         if not self.check_versions() or not self.check_model_type():
@@ -1077,19 +1102,24 @@ class AnalyzeCardList(SutekhPlugin):
         dCrypt = _get_back_counts(aCrypt)
         dLib = _get_back_counts(aLib)
         sText = "\t\t<b>Mixed Card Backs :</b>\n\n"
-        if dCrypt.has_key(None) and dLib.has_key(None):
+        # PDF trumps none
+        if 'PDF' in dCrypt or 'PDF' in dLib:
+            sText += ("Cards from a V:EKN released pdf set in the deck. "
+                      "The V:EKN requires that these cards be sleeved in "
+                      "opaque sleeves with a existing VtES card\n")
+        elif dCrypt.has_key(None) and dLib.has_key(None):
             sText += "Both library and crypt have cards with unspecified" \
                     " expansions.\nIgnoring the mixed backs.\n"
             return sText
-        # We know there are mixed cards, when we get here.
-        sText += "Mixed card backs in the deck. The V:EKN rules require " \
-                "that a mixed deck be of 'sufficiently mixed card types' " \
-                "if it's to be played without sleeves. This tests some " \
-                "obvious cases, but check with the event judge if playing " \
-                "without sleeves\n\n"
+        else:
+            # We know there are mixed cards, when we get here.
+            sText += ("Mixed card backs in the deck. The V:EKN rules require "
+                      "that a mixed deck be of 'sufficiently mixed card "
+                      "types' if it's to be played without sleeves. This "
+                      "tests some obvious cases, but check with the event "
+                      "judge if playing without sleeves\n\n")
         sText += self._check_crypt_backs(dCrypt, aCrypt)
         sText += self._check_lib_backs(dLib, aLib)
-
         return sText
 
     def happy_families_init(self, oHFVBox, oDlg):
